@@ -1,3 +1,7 @@
+#! /bin/sh
+# the next line restarts with tclsh \
+exec tclsh8.5 "$0" ${1+"$@"}
+
 # MAIN PROGRAM
 #
 # Usage:
@@ -8,10 +12,6 @@
 #	in a format similar to 'patch'. It not the same as patch, but could
 #	be modified to be exactly the same.
 
-lappend auto_path \
-	[file join \
-	[file dirname [file dirname [file dirname [file dirname [file join [pwd] [info script]]]]]] \
-	modules struct]
 package require struct
 
 # Open the files and read the lines into memory
